package org.jeecg.modules.wps.controller;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.wps.entity.WpsFile;
import org.jeecg.modules.wps.service.IWpsFileService;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

 /**
 * wps文件
 * @author jeecg-boot
 * @date   2020-06-01
 * @version V1.0
 */
@Slf4j
@Api(tags="wps文件")
@RestController
@RequestMapping("/ctop/wpsFile")
public class WpsFileController {
	@Autowired
	private IWpsFileService wpsFileService;

	/**
	  * 分页列表查询
	 * @param wpsFile
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@AutoLog(value = "wps文件-分页列表查询")
	@ApiOperation(value="wps文件-分页列表查询", notes="wps文件-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<WpsFile>> queryPageList(WpsFile wpsFile,
												@RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
												@RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
												HttpServletRequest req) {
		Result<IPage<WpsFile>> result = new Result<>();
		QueryWrapper<WpsFile> queryWrapper = QueryGenerator.initQueryWrapper(wpsFile, req.getParameterMap());
		Page<WpsFile> page = new Page<WpsFile>(pageNo, pageSize);
		IPage<WpsFile> pageList = wpsFileService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	/**
	  *   添加
	 * @param wpsFile
	 * @return
	 */
	@AutoLog(value = "wps文件-添加")
	@ApiOperation(value="wps文件-添加", notes="wps文件-添加")
	@PostMapping(value = "/add")
	public Result<WpsFile> add(@RequestBody WpsFile wpsFile) {
		Result<WpsFile> result = new Result<>();
		try {
			wpsFileService.save(wpsFile);
			result.success("添加成功！");
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
		}
		return result;
	}

	/**
	  *  编辑
	 * @param wpsFile
	 * @return
	 */
	@AutoLog(value = "wps文件-编辑")
	@ApiOperation(value="wps文件-编辑", notes="wps文件-编辑")
	@PutMapping(value = "/edit")
	public Result<WpsFile> edit(@RequestBody WpsFile wpsFile) {
		Result<WpsFile> result = new Result<WpsFile>();
		WpsFile wpsFileEntity = wpsFileService.getById(wpsFile.getId());
		if(wpsFileEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = wpsFileService.updateById(wpsFile);
			if(ok) {
				result.success("修改成功!");
			}
		}

		return result;
	}

	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@AutoLog(value = "wps文件-通过id删除")
	@ApiOperation(value="wps文件-通过id删除", notes="wps文件-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<?> delete(@RequestParam(name="id",required=true) String id) {
		try {
			wpsFileService.removeById(id);
		} catch (Exception e) {
			log.error("删除失败",e.getMessage());
			return Result.error("删除失败!");
		}
		return Result.ok("删除成功!");
	}

	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "wps文件-批量删除")
	@ApiOperation(value="wps文件-批量删除", notes="wps文件-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<WpsFile> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<WpsFile> result = new Result<>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.wpsFileService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}

	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@AutoLog(value = "wps文件-通过id查询")
	@ApiOperation(value="wps文件-通过id查询", notes="wps文件-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<WpsFile> queryById(@RequestParam(name="id",required=true) String id) {
		Result<WpsFile> result = new Result<>();
		WpsFile wpsFile = wpsFileService.getById(id);
		if(wpsFile==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(wpsFile);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<WpsFile> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              WpsFile wpsFile = JSON.parseObject(deString, WpsFile.class);
              queryWrapper = QueryGenerator.initQueryWrapper(wpsFile, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<WpsFile> pageList = wpsFileService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "wps文件列表");
      mv.addObject(NormalExcelConstants.CLASS, WpsFile.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("wps文件列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<WpsFile> listWpsFiles = ExcelImportUtil.importExcel(file.getInputStream(), WpsFile.class, params);
              wpsFileService.saveBatch(listWpsFiles);
              return Result.ok("文件导入成功！数据行数:" + listWpsFiles.size());
          } catch (Exception e) {
              log.error(e.getMessage(),e);
              return Result.error("文件导入失败:"+e.getMessage());
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

}
